/*******************************************************************************
* Copyright (C) 2013 Spansion LLC. All Rights Reserved. 
*
* This software is owned and published by: 
* Spansion LLC, 915 DeGuigne Dr. Sunnyvale, CA  94088-3453 ("Spansion").
*
* BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND 
* BY ALL THE TERMS AND CONDITIONS OF THIS AGREEMENT.
*
* This software contains source code for use with Spansion 
* components. This software is licensed by Spansion to be adapted only 
* for use in systems utilizing Spansion components. Spansion shall not be 
* responsible for misuse or illegal use of this software for devices not 
* supported herein.  Spansion is providing this software "AS IS" and will 
* not be responsible for issues arising from incorrect user implementation 
* of the software.  
*
* SPANSION MAKES NO WARRANTY, EXPRESS OR IMPLIED, ARISING BY LAW OR OTHERWISE,
* REGARDING THE SOFTWARE (INCLUDING ANY ACOOMPANYING WRITTEN MATERIALS), 
* ITS PERFORMANCE OR SUITABILITY FOR YOUR INTENDED USE, INCLUDING, 
* WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY, THE IMPLIED 
* WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE OR USE, AND THE IMPLIED 
* WARRANTY OF NONINFRINGEMENT.  
* SPANSION SHALL HAVE NO LIABILITY (WHETHER IN CONTRACT, WARRANTY, TORT, 
* NEGLIGENCE OR OTHERWISE) FOR ANY DAMAGES WHATSOEVER (INCLUDING, WITHOUT 
* LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS, BUSINESS INTERRUPTION, 
* LOSS OF BUSINESS INFORMATION, OR OTHER PECUNIARY LOSS) ARISING FROM USE OR 
* INABILITY TO USE THE SOFTWARE, INCLUDING, WITHOUT LIMITATION, ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL OR CONSEQUENTIAL DAMAGES OR LOSS OF DATA, 
* SAVINGS OR PROFITS, 
* EVEN IF SPANSION HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. 
* YOU ASSUME ALL RESPONSIBILITIES FOR SELECTION OF THE SOFTWARE TO ACHIEVE YOUR
* INTENDED RESULTS, AND FOR THE INSTALLATION OF, USE OF, AND RESULTS OBTAINED 
* FROM, THE SOFTWARE.  
*
* This software may be replicated in part or whole for the licensed use, 
* with the restriction that this Disclaimer and Copyright notice must be 
* included with each copy of this software, whether used in part or whole, 
* at all times.  
*/
/******************************************************************************/
/** \file main.c
 **
 ** This example demonstrates the calibration funciton of Real Time Clock. The
 ** original 1Hz signal divided by subclock is output from SUBOUT_0, and the 
 ** CO signal which has been calibrated with frequency correction module is 
 ** output from RTCCO_1.
 ** By press the SW3, the calibration step can be regulated.
 ** A frequency tester with high accracy can be used to monitor the output 
 ** change from RTCCO_1 when implementing the calibration function. 
 **
 ** History:
 **   - 2014-03-10  0.1  Edison Zhang         First version.
 **
 ******************************************************************************/

/******************************************************************************/
/* Include files                                                              */
/******************************************************************************/
#include "pdl_header.h"

/*****************************************************************************/
/* Local pre-processor symbols/macros ('#define')                            */
/*****************************************************************************/
/** 
  * As the calibration module can only slow the sub clock frequency by masking the 
  * clock pulses, always set the prescalor less than 32768
  */
#define RTC_CLOCK_PRES                  32760
/** 
  * Set the calibration cycle to 20s
  */
#define RTC_CLOCK_CALIBRATION_CYCLE     20
/** 
  *  Assume standard frequency of subclock is 32768Hz, the actual minimum
  *  frequency of sub clock is 32760Hz. So the deviation is 
  * (32768-32760)/32760 = 244.2ppm                                        
  * To calibrate 1 step can regulate 1.526ppm, if the calibraiton cycle is 20s
  * 1/32768*20 = 1.526ppm 
  * The initial calibration step count is 160
  * 244.2/1.526 = 160 steps 
  */
#define RTC_CAL_CENTER_VAL              160 

/*****************************************************************************/
/* Global variable definitions (declared in header file with 'extern')       */
/*****************************************************************************/

/*****************************************************************************/
/* Local type definitions ('typedef')                                        */
/*****************************************************************************/

/*****************************************************************************/
/* Local function prototypes ('static')                                      */
/*****************************************************************************/

/*****************************************************************************/
/* Local variable definitions ('static')                                     */
/*****************************************************************************/

/*****************************************************************************/
/* Function implementation - global ('extern') and local ('static')          */
/*****************************************************************************/
/**
 ******************************************************************************
 ** \brief  Time delay function
 ******************************************************************************/
static void Delay(uint32_t u32Cnt)
{
    uint32_t i;
    for(;u32Cnt;u32Cnt--)
        for(i=1000; i; i--);
}

static void ConfigRtc(void)
{
    en_result_t enResult;
    stc_rtc_config_t stcRtcConfig;
    stc_rtc_int_sel_t stcInt;
    stc_rtc_int_cb_t  stcIntCb;
    
    // Clear structures
    PDL_ZERO_STRUCT(stcRtcConfig);
    PDL_ZERO_STRUCT(stcInt);
    PDL_ZERO_STRUCT(stcIntCb);

    // Select clock
    stcRtcConfig.enClkSel = RtcUseSubClk;
    
    // Initialize RTCCO
    SetPinFunc_RTCCO_1();
    stcRtcConfig.enRtccoSel = RtccoOutput1Hz;
    
    // Initialize SUBOUT
    SetPinFunc_SUBOUT_0();
    stcRtcConfig.bEnSuboutDivider = TRUE;
    stcRtcConfig.enDividerRatio = RtcDivRatio32768;
    
    // Frequency correction setting
    stcRtcConfig.bInitFreqCorr = TRUE;
    stcRtcConfig.stcFreqCorrConfig.u16FreqCorrValue = RTC_CAL_CENTER_VAL;
    stcRtcConfig.stcFreqCorrConfig.u16FreqCorrCycle = RTC_CLOCK_CALIBRATION_CYCLE;
    
    // Time setting (23:59:00 1st of January 2014)
    stcRtcConfig.bInitTimeDate = TRUE;
    stcRtcConfig.stcTimeDate.u8Second = 0;                    // Second      : 00
    stcRtcConfig.stcTimeDate.u8Minute = 59;                   // Minutes     : 59
    stcRtcConfig.stcTimeDate.u8Hour   = 23;                   // Hour        : 23
    stcRtcConfig.stcTimeDate.u8Day    = 31;                   // Date        : 31th
    stcRtcConfig.stcTimeDate.u8Month  = RtcJanuary;           // Month       : January
    stcRtcConfig.stcTimeDate.u16Year   = 2014;                // Year        : 2014
    (void)Rtc_SetDayOfWeek(&stcRtcConfig.stcTimeDate);        // Set Day of the Week in stcRtcTime

    // Don't intialize RTC alarm
    stcRtcConfig.bInitAlarm = FALSE;

    // Don't intialize RTC timer
    stcRtcConfig.bInitTimer = FALSE;
    
    // Initialize the RTC
    enResult = Rtc_Init(&stcRtcConfig);

    if (Ok != enResult)
    {
#ifdef DEBUG_PRINT
        printf("Initial error!\n");
#endif
        while(1);
    }
    
    // Start RTC counting 
    Rtc_EnableFunc(RtcCount);
    
    // Start calibration
    Rtc_EnableFunc(RtcFreqCorr);
}

/**
 ******************************************************************************
 ** \brief  Main function of project for MB9BF56X series (FM0P).
 **
 ** \return uint32_t return value, if needed
 ******************************************************************************/
int32_t main(void)
{
    uint16_t u16CalVal;
    u16CalVal = RTC_CAL_CENTER_VAL;
#ifdef DEBUG_PRINT
    printf("***********************************************************\n");
    printf("*   S6E1A1 RTC Calibration test                           *\n");
    printf("*   Output 1Hz signal (before calibration) from SUBOUT_0  *\n");
    printf("*   Output 1Hz signal (after calibration) from RTCCO_1    *\n");
    printf("***********************************************************\n");
#endif 
    
    // Initialize GPIO for Key
    Gpio1pin_InitIn(GPIO1PIN_P04, Gpio1pin_InitPullup(1u));
    
    // Configure RTC
    ConfigRtc();
    
    while (1)
    {
        /* Incease calibration value if press key */
        if(FALSE == Gpio1pin_Get(GPIO1PIN_P04))
        {
            Delay(1000);
            u16CalVal++;
            if(u16CalVal > 1023)
            {
                u16CalVal = 0;
            }
            Rtc_SetFreqCorrValue(u16CalVal);
        #ifdef DEBUG_PRINT
            printf("Calibration value changes to 0x%04x\n", u16CalVal);
            printf("Check the RTCCO output\n");
        #endif
        }
    }
}

/******************************************************************************/
/* EOF (not truncated)                                                        */
/******************************************************************************/
