/*******************************************************************************
* Copyright (C) 2013 Spansion LLC. All Rights Reserved.
*
* This software is owned and published by:
* Spansion LLC, 915 DeGuigne Dr. Sunnyvale, CA  94088-3453 ("Spansion").
*
* BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND
* BY ALL THE TERMS AND CONDITIONS OF THIS AGREEMENT.
*
* This software contains source code for use with Spansion
* components. This software is licensed by Spansion to be adapted only
* for use in systems utilizing Spansion components. Spansion shall not be
* responsible for misuse or illegal use of this software for devices not
* supported herein.  Spansion is providing this software "AS IS" and will
* not be responsible for issues arising from incorrect user implementation
* of the software.
*
* SPANSION MAKES NO WARRANTY, EXPRESS OR IMPLIED, ARISING BY LAW OR OTHERWISE,
* REGARDING THE SOFTWARE (INCLUDING ANY ACOOMPANYING WRITTEN MATERIALS),
* ITS PERFORMANCE OR SUITABILITY FOR YOUR INTENDED USE, INCLUDING,
* WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY, THE IMPLIED
* WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE OR USE, AND THE IMPLIED
* WARRANTY OF NONINFRINGEMENT.
* SPANSION SHALL HAVE NO LIABILITY (WHETHER IN CONTRACT, WARRANTY, TORT,
* NEGLIGENCE OR OTHERWISE) FOR ANY DAMAGES WHATSOEVER (INCLUDING, WITHOUT
* LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS, BUSINESS INTERRUPTION,
* LOSS OF BUSINESS INFORMATION, OR OTHER PECUNIARY LOSS) ARISING FROM USE OR
* INABILITY TO USE THE SOFTWARE, INCLUDING, WITHOUT LIMITATION, ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL OR CONSEQUENTIAL DAMAGES OR LOSS OF DATA,
* SAVINGS OR PROFITS,
* EVEN IF SPANSION HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* YOU ASSUME ALL RESPONSIBILITIES FOR SELECTION OF THE SOFTWARE TO ACHIEVE YOUR
* INTENDED RESULTS, AND FOR THE INSTALLATION OF, USE OF, AND RESULTS OBTAINED
* FROM, THE SOFTWARE.
*
* This software may be replicated in part or whole for the licensed use,
* with the restriction that this Disclaimer and Copyright notice must be
* included with each copy of this software, whether used in part or whole,
* at all times.
*/
/******************************************************************************/
/** \file adc.h
 **
 ** Headerfile for ADC functions
 **
 ** History:
 **   - 2014-01-14  1.0  Edison Zhang  First version.
 **   - 2014-03-06  1.1  Edison Zhang  Add the SCFD register address defnition
 **
 ******************************************************************************/

#ifndef __ADC_H__
#define __ADC_H__

/******************************************************************************/
/* Include files                                                              */
/******************************************************************************/
#include "mcu.h"
#include "pdl_user.h"

#if (defined(PDL_PERIPHERAL_ADC_ACTIVE))

/* C binding of definitions if building with C++ compiler */
#ifdef __cplusplus
extern "C"
{
#endif

/**
 ******************************************************************************
 ** \defgroup AdcGroup 12-Bit Analog Digital Converter (ADC)
 **
 ** Provided functions of ADC module:
 ** - Adc_Init()
 ** - Adc_DeInit()
 ** - Adc_Enable()
 ** - Adc_Disable()
 ** - Adc_SwTriggerScan()
 ** - Adc_StopScanRepeat()
 ** - Adc_SwTriggerPrio()
 ** - Adc_ForceStop()
 ** - Adc_EnableInt()
 ** - Adc_DisableInt()
 ** - Adc_GetIntFlag()
 ** - Adc_ClrIntFlag()
 ** - Adc_GetScanFifo()
 ** - Adc_ClrScanFifo()
 ** - Adc_GetStatus()
 ** - Adc_ExtractScanFifo()
 ** - Adc_GetPrioFifo()
 ** - Adc_ClrPrioFifo()
 ** - Adc_ExtractPrioFifo()
 ** - AdcIrqHandler()
 **
 ** Adc_Init() initializes an ADC instance according the configuration
 ** of the structure type #stc_adc_config_t. Adc_DeInit() de-initializes an
 ** ADC instance.
 **
 ** For using the ADC instnace Adc_Enable() can be used first to enable ADC
 ** operation.
 **
 ** Adc_EnableInt() is used to enable ADC interrupts selected by the structure
 ** type #stc_adc_int_sel_t, the interrupt callback functions can also be set
 ** according to structure type #stc_adc_int_callback_t.
 **
 ** Adc_Disable() is used to disable ADC interrupts also selected by the
 ** structure type #stc_adc_int_sel_t
 **
 ** Adc_SwTriggerScan() and Adc_SwTriggerPrio() can be used for software trigger
 ** start for scan and priority conversion.
 **
 ** Adc_ForceStop() stops both ADC scan and priority conversion forcely.
 **
 ** Adc_GetIntFlag() can get the interrupt flag in the interrupt-polling mode
 ** and Adc_ClrIntFlag() can clear the interrupt flag. The interrupt type can
 ** be selected by the enumeration type #en_adc_int_t.
 **
 ** Adc_GetStatus() return the status of ADC selected by the structure
 ** type #en_adc_status_t.
 **
 ** Adc_ClrScanFifo() and Adc_ClrPrioFifo() clear the FIFOs.
 **
 ** Adc_GetScanFifo() and Adc_GetPrioFifo() read out the FIFOs.
 **
 ** Adc_ExtractScanFifo() and Adc_ExtractPrioFifo() extracts 32-bits FIFO data.
 **
 ** AdcIrqHandler() is ADC IRQ handler function, which is called in the
 ** interrupts.c file.
 **
 ** \note The callback arguments for the scan and priority result point to
 ** the according FIFO registers of the recent instance. The application must
 ** take care of emptying these FIFOs. Either the FIFO depth of the
 ** configuration can be used for counting or the function
 ** Adc_GetStatus() can be used to check if the FIFO is empty or not.
 **
 ******************************************************************************/
//@{

/******************************************************************************
 * Global definitions
 ******************************************************************************/
#define stc_adcn_t   FM0P_ADC_TypeDef
#define ADC0       (*((volatile stc_adcn_t *) FM0P_ADC0_BASE))
#define ADC1       (*((volatile stc_adcn_t *) FM0P_ADC1_BASE))
#define ADC2       (*((volatile stc_adcn_t *) FM0P_ADC2_BASE))

#define PDL_ADC_READY_WAIT_COUNT 1000000u        ///< Time-out counter value for ADC ready bit polling

#define ADC_INSTANCE_COUNT                                \
        (uint8_t)(PDL_PERIPHERAL_ENABLE_ADC0 == PDL_ON) + \
        (uint8_t)(PDL_PERIPHERAL_ENABLE_ADC1 == PDL_ON) + \
        (uint8_t)(PDL_PERIPHERAL_ENABLE_ADC2 == PDL_ON)

#define ADC0_SCAN_FIFO_ADDR     (uint32_t)(&FM0P_ADC0->SCFD)
#define ADC1_SCAN_FIFO_ADDR     (uint32_t)(&FM0P_ADC1->SCFD)
#define ADC2_SCAN_FIFO_ADDR     (uint32_t)(&FM0P_ADC2->SCFD)

/******************************************************************************
 * Global type definitions
 ******************************************************************************/
typedef void (*func_ptr_adc_parg32_t)(volatile uint32_t* pu32Argument);

/**
 ******************************************************************************
 ** \brief Scan Conversion Mode
 ******************************************************************************/
typedef enum en_adc_scan_mode
{
    ScanSingleConversion  = 0,        ///< Single mode of scan conversion
    ScanRepeatConversion  = 1         ///< Repeat mode of scan conversion
} en_adc_scan_mode_t;

/**
 ******************************************************************************
 ** \brief Priority Conversion Stage Count
 ******************************************************************************/
typedef enum en_adc_sample_time_n
{
    Value1        = 0,    ///< Set value * 1
    Value4        = 1,    ///< Set value * 4
    Value8        = 2,    ///< Set value * 8
    Value16       = 3,    ///< Set value * 16
    Value32       = 4,    ///< Set value * 32
    Value64       = 5,    ///< Set value * 64
    Value128      = 6,    ///< Set value * 128
    Value256      = 7,    ///< Set value * 256
} en_adc_sample_time_n_t;

/**
 ******************************************************************************
 ** \brief FIFO Status (Scan, Priority Conversion)
 ******************************************************************************/
typedef enum en_adc_fifo_status
{
    AdcFifoEmpty     = 0,    ///< Scan FIFO empty
    AdcFifoFilled    = 1,    ///< Scan FIFO (partly) filled
    AdcFifoFull      = 2,    ///< Scan FIFO full
    AdcFifoOverrun   = 3,    ///< Scan FIFO overrun
    AdcFifoError     = 4     ///< Unknown Fifo Error (no bit set) or pstcAdc == NULL
} en_adc_fifo_status_t;

/**
 ******************************************************************************
 ** \brief ADC scan/priority conversion timer trigger selection
 ** \note Do not change the numbering.
 ******************************************************************************/
typedef enum en_adc_timer_select
{
    AdcNoTimer =  0,   ///< No selected trigger
    AdcMft     =  1,   ///< Trigger by Multifunction Timer
    AdcBt0     =  2,   ///< Trigger with Base Timer 0
    AdcBt1     =  3,   ///< Trigger with Base Timer 1
    AdcBt2     =  4,   ///< Trigger with Base Timer 2
    AdcBt3     =  5,   ///< Trigger with Base Timer 3
    AdcBt4     =  6,   ///< Trigger with Base Timer 4
    AdcBt5     =  7,   ///< Trigger with Base Timer 5
    AdcBt6     =  8,   ///< Trigger with Base Timer 6
    AdcBt7     =  9,   ///< Trigger with Base Timer 7

} en_adc_timer_select_t;

/**
 ******************************************************************************
 ** \brief ADC Channel list with same sort order as the ER32 registers
 ******************************************************************************/
typedef struct stc_ad_channel_list
{
    union unAdcChannels
    {
        uint32_t u32AD_CHn;
        struct stcChannelBits
        {
            uint32_t AD_CH_0  : 1 ;
            uint32_t AD_CH_1  : 1 ;
            uint32_t AD_CH_2  : 1 ;
            uint32_t AD_CH_3  : 1 ;
            uint32_t AD_CH_4  : 1 ;
            uint32_t AD_CH_5  : 1 ;
            uint32_t AD_CH_6  : 1 ;
            uint32_t AD_CH_7  : 1 ;
            uint32_t AD_CH_8  : 1 ;
            uint32_t AD_CH_9  : 1 ;
            uint32_t AD_CH_10 : 1 ;
            uint32_t AD_CH_11 : 1 ;
            uint32_t AD_CH_12 : 1 ;
            uint32_t AD_CH_13 : 1 ;
            uint32_t AD_CH_14 : 1 ;
            uint32_t AD_CH_15 : 1 ;
            uint32_t AD_CH_16 : 1 ;
            uint32_t AD_CH_17 : 1 ;
            uint32_t AD_CH_18 : 1 ;
            uint32_t AD_CH_19 : 1 ;
            uint32_t AD_CH_20 : 1 ;
            uint32_t AD_CH_21 : 1 ;
            uint32_t AD_CH_22 : 1 ;
            uint32_t AD_CH_23 : 1 ;
            uint32_t AD_CH_24 : 1 ;
            uint32_t AD_CH_25 : 1 ;
            uint32_t AD_CH_26 : 1 ;
            uint32_t AD_CH_27 : 1 ;
            uint32_t AD_CH_28 : 1 ;
            uint32_t AD_CH_29 : 1 ;
            uint32_t AD_CH_30 : 1 ;
            uint32_t AD_CH_31 : 1 ;
        };
    };
} stc_ad_channel_list_t ;

/**
 ******************************************************************************
 ** \brief Scan Conversion configuration
 ******************************************************************************/
typedef struct stc_adc_scan
{
    en_adc_scan_mode_t            enScanMode;                     ///< Conversion Mode. See #en_adc_scan_mode_t for details
    stc_ad_channel_list_t         u32ScanCannelSelect;            ///< Selects scan channels for ADC (1 = active)
    boolean_t                     bScanTimerStartEnable;          ///< TRUE: Triggers Scan Conversion by Timer
    en_adc_timer_select_t         enScanTimerTrigger;             ///< See #en_adc_timer_select_t for details
    uint8_t                       u8ScanFifoDepth;                ///< Depth of the Scan Conversion FIFO 0...15

}stc_adc_scan_t;

/**
 ******************************************************************************
 ** \brief Priority Conversion configuration
 ******************************************************************************/
typedef struct stc_adc_prio
{
    boolean_t                     bPrioExtTrigStartEnable;        ///< TRUE: Triggers Prioritys Conversion on falling edge of external signal
    boolean_t                     bPrioTimerStartEnable;          ///< TRUE: Triggers Priority Conversion by Timer
    en_adc_timer_select_t         enPrioTimerTrigger;             ///< See #en_adc_timer_select_t for details
    uint8_t                       u8PrioFifoDepth;                ///< Depth of the Priority Conversion FIFO 0...3
    uint8_t                       u8PrioLevel1AnalogChSel;        ///< Priority Level 1 Analog Channel Selector Ch. 0...7
    uint8_t                       u8PrioLevel2AnalogChSel;        ///< Priority Level 2 Analog Channel Selector Ch. 0...31

}stc_adc_prio_t;

/**
 ******************************************************************************
 ** \brief Comparison configuration
 ******************************************************************************/
typedef struct stc_adc_comapre
{
    uint16_t                      u16CompareValue;                ///< ADC Comparison Value (CMPD)
    boolean_t                     bCompareAllChannels;            ///< TRUE: Compare all selected Channels, FALSE: Compare CCH-Channel
    uint8_t                       u8CompareChannel;               ///< CCH-Channel to be compared, if selected Ch. 0...31
    boolean_t                     bCompIrqEqualGreater;           ///< TRUE: Generate Interrupt, if CMPD most significant 10 bis >= current ADC value
                                                                  ///< FALSE: Generate Interrupt, if CMPD most significant 10 bis < current ADC value
}stc_adc_comapre_t;

/**
 ******************************************************************************
 ** \brief Range comparison configuration
 ******************************************************************************/
typedef struct stc_adc_range_compare
{
    uint16_t                      u16UpperLimitRangeValue;        ///< Upper Limit Value for Range Comparison
    uint16_t                      u16LowerLimitRangeValue;        ///< Lower limit Value for Range Comparison
    uint8_t                       u8RangeCountValue;              ///< Range Count Value 1...7
    boolean_t                     bWithinRange;                   ///< TRUE: Value within Range, FALSE: Value out of Range
    boolean_t                     bRangeCompareAllChannels;       ///< TRUE: Range compare all selected Channels, FALSE: Compare WCCH-Channel
    uint8_t                       u8RangeComapreChannel;          ///< WCCH-Channel to be Range compared, if selected Ch. 0...31

}stc_adc_range_compare_t;

/**
 ******************************************************************************
 ** \brief ADC configuration.
 ******************************************************************************/
typedef struct stc_adc_config
{
    boolean_t                     bLsbAlignment;                  ///< TURE: Result is LSB aligned
    stc_ad_channel_list_t         u32SamplingTimeSelect;          ///< Selects channels for Sampling time 0 or 1 setting
    en_adc_sample_time_n_t        enSamplingTimeN0;               ///< Sampling Time N0 Time Setting Bits, see #en_adc_sample_time_n_t for details
    uint8_t                       u8SamplingTime0;                ///< Sampling Time 0 0...15
    en_adc_sample_time_n_t        enSamplingTimeN1;               ///< Sampling Time N1 Time Setting Bits, see #en_adc_sample_time_n_t for details
    uint8_t                       u8SamplingTime1;                ///< Sampling Time 1 0...15
    uint8_t                       u8SamplingMultiplier;           ///< Multiplier of N, see Peripheral Manual for Details!
    uint8_t                       u8EnableTime;                   ///< Enable Time

    boolean_t                     bScanInit;                      ///< TRUE: initialize scan conversion
    stc_adc_scan_t                stcScanInit;                    ///< Scan conversion configuration structure, it is available to configure scan coversion only when bScanInit is set to TRUE.

    boolean_t                     bPrioInit;                      ///< TRUE: initialize scan conversion
    stc_adc_prio_t                stcPrioInit;                    ///< Priority conversion configuration structure, it is available only when bPrioInit is set to TRUE.

    boolean_t                     bComparisonEnable;              ///< TRUE: Enables Comparison Mode
    stc_adc_comapre_t             stcComparisonInit;              ///< Comparison structure, it is available to configure comparison function only when bCompareEnable is set to TRUE

    boolean_t                     bRangeComparisonEnable;         ///< TRUE: Initialize and enable the Range Comparison
    stc_adc_range_compare_t       stcRangeComparisonInit;         ///< Range comparison structure, it is available to configure range comparison function only when bRangeCompareEnable is set to TRUE

} stc_adc_config_t;

/**
 ******************************************************************************
 ** \brief ADC interrupt selection.
 ******************************************************************************/
typedef struct stc_adc_int_sel
{
    boolean_t                     bScanIrq;                 ///< TRUE: Enable Scan Conversion Interrupt
    boolean_t                     bPrioIrq;                 ///< TRUE: Enable Priority Conversion Interrupt
    boolean_t                     bFifoOverrunIrq;          ///< TRUE: Enable FIFO Overrun Interrupt
    boolean_t                     bComparisonIrq;           ///< TRUE: Enable Conversion Comparison Interrupt
    boolean_t                     bRangeComparisonIrq;      ///< TRUE: Enable Range Comparison Interrupt

}stc_adc_int_sel_t;

/**
 ******************************************************************************
 ** \brief Scan conversion start cause
 ******************************************************************************/
typedef enum en_scan_start_cause
{
    ScanStartBySw        = 0,  ///< Scan converisons triggered by software
    ScanStartByTimer     = 1,  ///< Scan converisons triggered by timer

}en_scan_start_cause_t;

/**
 ******************************************************************************
 ** \brief Scan conversion FIFO information
 ******************************************************************************/
typedef struct stc_scan_fifo_info
{
    uint16_t                u16Data;      ///< Scan converison result data
    boolean_t               bInValid;     ///< Scan converison invalid or not, TRUE: data valid, FALSE: data invalid
    en_scan_start_cause_t   enStartCause; ///< Scan converison start cause
    uint8_t                 u8Ch;         ///< Scan converison channel which is sampled

}stc_scan_fifo_info_t;

/**
 ******************************************************************************
 ** \brief Priority conversion start cause
 ******************************************************************************/
typedef enum en_prio_start_cause
{
    PrioStartBySw         = 0,   ///< Priority conversion started by software
    PrioStartByTimer      = 1,   ///< Priority conversion started by timer
    PrioStartByExtTrig    = 2,   ///< Priority conversion started by external trigger

}en_prio_start_cause_t;

/**
 ******************************************************************************
 ** \brief Priority conversion FIFO information
 ******************************************************************************/
typedef struct stc_prio_fifo_info
{
    uint16_t                u16Data;       ///< Priority conversion result data
    boolean_t               bInValid;      ///< Priority conversion invalid or not, TRUE: data valid, FALSE: data invalid
    en_prio_start_cause_t   enStartCause;  ///< Priority conversion start cause
    uint8_t                 u8Ch;          ///< Priority conversion channel which is sampled

}stc_prio_fifo_info_t;

/**
 ******************************************************************************
 ** \brief ADC interrupt callback function.
 ******************************************************************************/
typedef struct stc_adc_int_callback
{
    func_ptr_adc_parg32_t         pfnScanCallback;              ///< Scanconversion callback
    func_ptr_adc_parg32_t         pfnPrioCallback;              ///< Priority conversion callback
    func_ptr_t                    pfnScanErrorCallback;         ///< Scan FIFO overrun error during conversion
    func_ptr_t                    pfnPrioErrorCallback;         ///< Priority FIFO overrun error callback pointer
    func_ptr_t                    pfnComparisonCallback;        ///< Comparison callback pointer
    func_ptr_t                    pfnRangeComparisonCallback;   ///< Range condition interrupt callback pointer

}stc_adc_int_callback_t;

/**
 ******************************************************************************
 ** \brief ADC status list
 ******************************************************************************/
typedef enum en_adc_status
{
    ScanFifoEmptyStatus         = 0,  ///< Scan FIFO empty status
    ScanFifoFullStatus          = 1,  ///< Scan FIFO full status
    PrioFifoEmptyStatus         = 2,  ///< Priority FIFO empty status
    PrioFifoFullStatus          = 3,  ///< Priority FIFO full status
    Prio2PendingStatus          = 4,  ///< Priority conversion (level 2) pending status
    PrioStatus                  = 5,  ///< Priority conversion status
    ScanStatus                  = 6,  ///< Scan conversion status
    RangeThresholdExcessFlag    = 7,  ///< Range comparison threshold exceed status

}en_adc_status_t;

/**
 ******************************************************************************
 ** \brief ADC interrupt type
 ******************************************************************************/
typedef enum en_adc_int
{
    AdcScanInt                = 0,   ///< Scan conversion interrupt
    AdcPrioInt                = 1,   ///< Priority conversion interrupt
    AdcScanFifoOverrunInt     = 2,   ///< Scan FIFO overrun interrupt
    AdcPrioFifoOverrunInt     = 3,   ///< Priority FIFO overrun interrupt
    AdcComparisonInt          = 4,   ///< ADC comparison interrupt
    AdcRangeComparisonInt     = 5,   ///< ADC range comparison interrupt

}en_adc_int_t;

/// Enumeration to define an index for each enabled ADC instance
typedef enum en_adc_instance_index
{
  #if (PDL_PERIPHERAL_ENABLE_ADC0 == PDL_ON)
    AdcInstanceIndexAdc0,
  #endif
  #if (PDL_PERIPHERAL_ENABLE_ADC1 == PDL_ON)
    AdcInstanceIndexAdc1,
  #endif
  #if (PDL_PERIPHERAL_ENABLE_ADC2 == PDL_ON)
    AdcInstanceIndexAdc2,
  #endif
    AdcInstanceIndexMax
} en_adc_instance_index_t;

/// ADC instance internal data, storing internal information for each enabled ADC instance.
typedef struct stc_adc_intern_data
{
  func_ptr_adc_parg32_t   pfnScanCallback;              ///< Scan conversion callback
  func_ptr_adc_parg32_t   pfnPrioCallback;              ///< Priority conversion callback
  func_ptr_t              pfnScanErrorCallback;                 ///< Error during conversion
  func_ptr_t              pfnPrioErrorCallback;                 ///< Priority FIFO overrun error callback pointer
  func_ptr_t              pfnComparisonCallback;                ///< Comparison callback pointer
  func_ptr_t              pfnRangeComparisonCallback;           ///< Range condition interrupt callback pointer
} stc_adc_intern_data_t;

/// ADC instance data type
typedef struct stc_adc_instance_data
{
  volatile stc_adcn_t*  pstcInstance;  ///< pointer to registers of an instance
  stc_adc_intern_data_t stcInternData; ///< module internal data of instance
} stc_adc_instance_data_t;

/******************************************************************************/
/* Global variable declarations ('extern', definition in C source)            */
/******************************************************************************/

/// Look-up table for all enabled ADC instances and their internal data
extern stc_adc_instance_data_t m_astcAdcInstanceDataLut[ADC_INSTANCE_COUNT];

/******************************************************************************/
/* Global function prototypes ('extern', definition in C source)              */
/******************************************************************************/
/* Init/De-Init */
en_result_t               Adc_Init( volatile stc_adcn_t* pstcAdc,
                                    stc_adc_config_t*    pstcConfig );
en_result_t               Adc_DeInit( volatile stc_adcn_t* pstcAdc );

#if (PDL_INTERRUPT_ENABLE_ADC0 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_ADC1 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_ADC2 == PDL_ON)
/* Int Enable/Disable */
en_result_t               Adc_EnableInt(volatile stc_adcn_t* pstcAdc,
                                        stc_adc_int_sel_t* pstcIntSel,
                                        stc_adc_int_callback_t* pstcIntCallback);
en_result_t               Adc_DisableInt(volatile stc_adcn_t* pstcAdc,
                                         stc_adc_int_sel_t* pstcIntSel);
#endif
/* Int Flag get/clear */
boolean_t                 Adc_GetIntFlag(volatile stc_adcn_t* pstcAdc,
                                         en_adc_int_t enInt);
en_result_t               Adc_ClrIntFlag(volatile stc_adcn_t* pstcAdc,
                                         en_adc_int_t enInt);

/* Function enable/disable */
en_result_t               Adc_Enable( volatile stc_adcn_t* pstcAdc );
en_result_t               Adc_Disable( volatile stc_adcn_t* pstcAdc );
en_result_t               Adc_SwTriggerScan( volatile stc_adcn_t* pstcAdc );
en_result_t               Adc_StopScanRepeat( volatile stc_adcn_t* pstcAdc );
en_result_t               Adc_SwTriggerPrio( volatile stc_adcn_t* pstcAdc );
en_result_t               Adc_ForceStop( volatile stc_adcn_t* pstcAdc );
/* Status */
boolean_t                 Adc_GetStatus(volatile stc_adcn_t* pstcAdc,
                                        en_adc_status_t enStatus);
/* FIFO data get/clear */
uint32_t                  Adc_GetScanFifo(volatile stc_adcn_t* pstcAdc);
en_result_t               Adc_ClrScanFifo(volatile stc_adcn_t* pstcAdc);
en_result_t               Adc_ExtractScanFifo(volatile stc_adcn_t* pstcAdc,
                                              uint32_t u32FifoData,
                                              stc_scan_fifo_info_t* pstcFifo);
uint32_t                  Adc_GetPrioFifo(volatile stc_adcn_t* pstcAdc);
en_result_t               Adc_ClrPrioFifo(volatile stc_adcn_t* pstcAdc);
en_result_t               Adc_ExtractPrioFifo(volatile stc_adcn_t* pstcAdc,
                                              uint32_t u32FifoData,
                                              stc_prio_fifo_info_t* pstcFifo);
/* ADC IRQ */
void                      AdcIrqHandler( volatile stc_adcn_t*   pstcAdc,
                                         stc_adc_intern_data_t* pstcAdcInternData );
//@}
#ifdef __cplusplus
}
#endif

#endif // #if (defined(PDL_PERIPHERAL_ADC_ACTIVE))

#endif /* __ADC_H__ */
/******************************************************************************/
/* EOF (not truncated)                                                        */
/******************************************************************************/
