/*******************************************************************************
* Copyright (C) 2013 Spansion LLC. All Rights Reserved. 
*
* This software is owned and published by: 
* Spansion LLC, 915 DeGuigne Dr. Sunnyvale, CA  94088-3453 ("Spansion").
*
* BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND 
* BY ALL THE TERMS AND CONDITIONS OF THIS AGREEMENT.
*
* This software contains source code for use with Spansion 
* components. This software is licensed by Spansion to be adapted only 
* for use in systems utilizing Spansion components. Spansion shall not be 
* responsible for misuse or illegal use of this software for devices not 
* supported herein.  Spansion is providing this software "AS IS" and will 
* not be responsible for issues arising from incorrect user implementation 
* of the software.  
*
* SPANSION MAKES NO WARRANTY, EXPRESS OR IMPLIED, ARISING BY LAW OR OTHERWISE,
* REGARDING THE SOFTWARE (INCLUDING ANY ACOOMPANYING WRITTEN MATERIALS), 
* ITS PERFORMANCE OR SUITABILITY FOR YOUR INTENDED USE, INCLUDING, 
* WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY, THE IMPLIED 
* WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE OR USE, AND THE IMPLIED 
* WARRANTY OF NONINFRINGEMENT.  
* SPANSION SHALL HAVE NO LIABILITY (WHETHER IN CONTRACT, WARRANTY, TORT, 
* NEGLIGENCE OR OTHERWISE) FOR ANY DAMAGES WHATSOEVER (INCLUDING, WITHOUT 
* LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS, BUSINESS INTERRUPTION, 
* LOSS OF BUSINESS INFORMATION, OR OTHER PECUNIARY LOSS) ARISING FROM USE OR 
* INABILITY TO USE THE SOFTWARE, INCLUDING, WITHOUT LIMITATION, ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL OR CONSEQUENTIAL DAMAGES OR LOSS OF DATA, 
* SAVINGS OR PROFITS, 
* EVEN IF SPANSION HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. 
* YOU ASSUME ALL RESPONSIBILITIES FOR SELECTION OF THE SOFTWARE TO ACHIEVE YOUR
* INTENDED RESULTS, AND FOR THE INSTALLATION OF, USE OF, AND RESULTS OBTAINED 
* FROM, THE SOFTWARE.  
*
* This software may be replicated in part or whole for the licensed use, 
* with the restriction that this Disclaimer and Copyright notice must be 
* included with each copy of this software, whether used in part or whole, 
* at all times.  
*/
/******************************************************************************/
/** \file lvd.c
 **
 ** A detailed description is available at 
 ** @link LvdGroup Low voltage Detection description @endlink
 **
 ** History:
 **   - 2014-02-03  1.0  Edison Zhang   First version.
 **
 ******************************************************************************/

/******************************************************************************/
/* Include files                                                              */
/******************************************************************************/
#include "lvd.h"

#if (defined(PDL_PERIPHERAL_LVD_ACTIVE))

/**
 ******************************************************************************
 ** \addtogroup LvdGroup
 ******************************************************************************/
//@{

/******************************************************************************/
/* Local pre-processor symbols/macros ('#define')                             */
/******************************************************************************/

/******************************************************************************/
/* Local type definitions ('typedef')                                         */
/******************************************************************************/

/******************************************************************************/
/* Local function prototypes ('static')                                       */
/******************************************************************************/

/******************************************************************************/
/* Local variable definitions ('static')                                      */
/******************************************************************************/
#if (PDL_INTERRUPT_ENABLE_LVD == PDL_ON)
static func_ptr_t  pfnLvdCallback;    ///< LVD interrupt callback function
#endif
/******************************************************************************/
/* Function implementation - global ('extern') and local ('static')           */
/******************************************************************************/
/* LVD enable timeout count */
#define LVD_TIMEOUT     (SystemCoreClock/10)

/* Unlock code for LVD_RLR */
#define LvdCtlUnlock    (0x1ACCE553)

#if (PDL_INTERRUPT_ENABLE_LVD == PDL_ON)
/**
 ******************************************************************************
 ** \brief ISR of the LVD module
 ** 
 ******************************************************************************/
void LvdIrqHandler(void)
{
    FM0P_LVD->LVD_CLR = 0x00;  /* Clear LVD interrupt (LVDCL (bit #7)) */

    if (NULL != pfnLvdCallback)
    {
        pfnLvdCallback();
    }
} /* LvdIrqHandler */

/**
 ******************************************************************************
 ** \brief Configure LVD interrupt
 **
 ** \param pfnCallback interrupt callback function 
 **
 ** \retval Ok LVD interrupt configured.
 **
 ** \note
 ** - If pfnCallback = NULL, disable the IRQ 
 ** - if pfnCallback != NULL, set the interrupt callback function and enable IRQ
 **
 ******************************************************************************/
en_result_t Lvd_ConfigInt(func_ptr_t pfnCallback)
{
    /* If callback function is set, interrupt is used. */
    if (NULL != pfnCallback)
    {
        pfnLvdCallback = pfnCallback;
      
        /* Init NVIC */
        NVIC_ClearPendingIRQ(LVD_IRQn);
        NVIC_EnableIRQ(LVD_IRQn);
        NVIC_SetPriority(LVD_IRQn, PDL_IRQ_LEVEL_LVD);

    }
    else
    {
        pfnLvdCallback = NULL;
      
        /* DeInit NVIC */
        NVIC_ClearPendingIRQ(LVD_IRQn);
        NVIC_DisableIRQ(LVD_IRQn);
        NVIC_SetPriority(LVD_IRQn, PDL_DEFAULT_INTERRUPT_LEVEL);
    }
    
    return Ok;
}
#endif

/**
 ******************************************************************************
 ** \brief Set the interrupt detect voltage of LVD
 **
 ** \param enVolt  interrupt detect voltage of LVD
 ** \arg LvdIntVoltage280    Set interrupt detect voltage to 2.80
 ** \arg LvdIntVoltage300    Set interrupt detect voltage to 3.00
 ** \arg LvdIntVoltage320    Set interrupt detect voltage to 3.20
 ** \arg LvdIntVoltage360    Set interrupt detect voltage to 3.60
 ** \arg LvdIntVoltage370    Set interrupt detect voltage to 3.70
 ** \arg LvdIntVoltage400    Set interrupt detect voltage to 4.00
 ** \arg LvdIntVoltage410    Set interrupt detect voltage to 4.10
 ** \arg LvdIntVoltage420    Set interrupt detect voltage to 4.20
 **
 ** \retval Ok       The interrupt detect voltage of LVD is set normally
 ** \retval ErrorInvalidParameter   Invalid setting of enVolt
 **
 ******************************************************************************/
en_result_t Lvd_SetIntDetectVoltage(en_lvd_int_volt_t enVolt)
{
    FM0P_LVD->LVD_RLR = LvdCtlUnlock;/* unlock LVD_CTL */   
    switch (enVolt)
    {
        case LvdIntVoltage280:  /* Interrupt when voltage is vicinity of 2.80 volts */
            FM0P_LVD->LVD_CTL_f.SVHI = 0x03u;
            break;
        case LvdIntVoltage300:  /* Interrupt when voltage is vicinity of 3.00 volts */
            FM0P_LVD->LVD_CTL_f.SVHI = 0x04u;
            break;
        case LvdIntVoltage320:  /* Interrupt when voltage is vicinity of 3.20 volts */
            FM0P_LVD->LVD_CTL_f.SVHI = 0x05u;
            break;
        case LvdIntVoltage360:  /* Interrupt when voltage is vicinity of 3.60 volts */
            FM0P_LVD->LVD_CTL_f.SVHI = 0x06u;
            break;
        case LvdIntVoltage370:  /* Interrupt when voltage is vicinity of 3.70 volts */
            FM0P_LVD->LVD_CTL_f.SVHI = 0x07u;
            break;
        case LvdIntVoltage400:  /* Interrupt when voltage is vicinity of 4.00 volts */
            FM0P_LVD->LVD_CTL_f.SVHI = 0x08u;
            break;
        case LvdIntVoltage410:  /* Interrupt when voltage is vicinity of 4.10 volts */
            FM0P_LVD->LVD_CTL_f.SVHI = 0x09u;
            break;
        case LvdIntVoltage420:  /* Interrupt when voltage is vicinity of 4.20 volts */
            FM0P_LVD->LVD_CTL_f.SVHI = 0x0Au;
            break;
        default:
            FM0P_LVD->LVD_RLR = 0;           /* lock LVD_CTL */
            return  ErrorInvalidParameter;
    }
    
    FM0P_LVD->LVD_RLR = 0;           /* lock LVD_CTL */
    
    return Ok;
}

/**
 ******************************************************************************
 ** \brief Set the reset detect voltage of LVD
 **
 ** \param enVolt  interrupt detect voltage of LVD
 ** \arg LvdResetVoltage245    Set reset detect voltage to 2.45
 ** \arg LvdResetVoltage260    Set reset detect voltage to 2.60
 ** \arg LvdResetVoltage270    Set reset detect voltage to 2.70
 ** \arg LvdResetVoltage280    Set reset detect voltage to 2.80
 ** \arg LvdResetVoltage300    Set reset detect voltage to 3.00
 ** \arg LvdResetVoltage320    Set reset detect voltage to 3.20
 ** \arg LvdResetVoltage360    Set reset detect voltage to 3.60
 ** \arg LvdResetVoltage370    Set reset detect voltage to 3.70
 ** \arg LvdResetVoltage400    Set reset detect voltage to 4.00
 ** \arg LvdResetVoltage410    Set reset detect voltage to 4.10
 ** \arg LvdResetVoltage420    Set reset detect voltage to 4.20
 **
 ** \retval Ok       The reset detect voltage of LVD is set normally
 ** \retval ErrorInvalidParameter   Invalid setting of enVolt
 **
 ******************************************************************************/
en_result_t Lvd_SetResetDetectVoltage(en_lvd_reset_volt_t enVolt)
{
    FM0P_LVD->LVD_RLR = LvdCtlUnlock;/* unlock LVD_CTL */            
    switch (enVolt)
    {
        case LvdResetVoltage245:  /* Interrupt when voltage is vicinity of 2.45 volts */
            FM0P_LVD->LVD_CTL_f.SVHR = 0x00u;
            break;
        case LvdResetVoltage260:  /* Interrupt when voltage is vicinity of 2.60 volts */
            FM0P_LVD->LVD_CTL_f.SVHR = 0x01u;
            break;
        case LvdResetVoltage270:  /* Interrupt when voltage is vicinity of 2.70 volts */
            FM0P_LVD->LVD_CTL_f.SVHR = 0x02u;
            break;
        case LvdResetVoltage280:  /* Interrupt when voltage is vicinity of 2.80 volts */
            FM0P_LVD->LVD_CTL_f.SVHR = 0x03u;
            break;
        case LvdResetVoltage300:  /* Interrupt when voltage is vicinity of 3.00 volts */
            FM0P_LVD->LVD_CTL_f.SVHR = 0x04u;
            break;
        case LvdResetVoltage320:  /* Interrupt when voltage is vicinity of 3.20 volts */
            FM0P_LVD->LVD_CTL_f.SVHR = 0x05u;
            break;
        case LvdResetVoltage360:  /* Interrupt when voltage is vicinity of 3.60 volts */
            FM0P_LVD->LVD_CTL_f.SVHR = 0x06u;
            break;
        case LvdResetVoltage370:  /* Interrupt when voltage is vicinity of 3.70 volts */
            FM0P_LVD->LVD_CTL_f.SVHR = 0x07u;
            break;
        case LvdResetVoltage400:  /* Interrupt when voltage is vicinity of 4.00 volts */
            FM0P_LVD->LVD_CTL_f.SVHR = 0x08u;
            break;
        case LvdResetVoltage410:  /* Interrupt when voltage is vicinity of 4.10 volts */
            FM0P_LVD->LVD_CTL_f.SVHR = 0x09u;
            break;
        case LvdResetVoltage420:  /* Interrupt when voltage is vicinity of 4.20 volts */
            FM0P_LVD->LVD_CTL_f.SVHR = 0x0Au;
            break;    
        default:
            FM0P_LVD->LVD_RLR = 0;           /* lock LVD_CTL */
            return  ErrorInvalidParameter;
    }
    
    FM0P_LVD->LVD_RLR = 0;           /* lock LVD_CTL */
    
    return Ok;
}

/**
 ******************************************************************************
 ** \brief Enable LVD function
 **
 ** \param enFunc      LVD function
 ** \arg LvdIntFunc    LVD interrupt detection function
 ** \arg LvdResetFunc  LVD reset detection function
 **
 ** \retval Ok       Enable LVD function normally
 ** \retval ErrorInvalidParameter   Invalid setting of enFunc
 **
 ******************************************************************************/
en_result_t Lvd_EnableFunc(en_lvd_func_t enFunc)
{
    uint32_t u32TimeOut = LVD_TIMEOUT;
    
    FM0P_LVD->LVD_RLR = LvdCtlUnlock;/* unlock LVD_CTL */     
    switch(enFunc)
    {
        case LvdIntFunc:
            FM0P_LVD->LVD_CTL_f.LVDIE = 1;
            while(1)
            {
                if(FM0P_LVD->LVD_STR2_f.LVDIRDY == 1)
                {
                    break;
                }
                
                u32TimeOut--;
                
                if(u32TimeOut == 0)
                {
                    return ErrorTimeout;
                }
            }

            break;
        case LvdResetFunc:
            FM0P_LVD->LVD_CTL_f.LVDRE = 1;
            while(1)
            {
                if(FM0P_LVD->LVD_STR2_f.LVDRRDY == 1)
                {
                    break;
                }
                
                u32TimeOut--;
                
                if(u32TimeOut == 0)
                {
                    return ErrorTimeout;
                }
            }
            break;  
        default:
            FM0P_LVD->LVD_RLR = 0;           /* lock LVD_CTL */
            return  ErrorInvalidParameter;
    }
    
    FM0P_LVD->LVD_RLR = 0;           /* lock LVD_CTL */
    
    return Ok;
}

/**
 ******************************************************************************
 ** \brief Disable LVD function
 **
 ** \param enFunc      LVD function
 ** \arg LvdIntFunc    LVD interrupt detection function
 ** \arg LvdResetFunc  LVD reset detection function
 **
 ** \retval Ok       Disable LVD function normally
 ** \retval ErrorInvalidParameter   Invalid setting of enFunc
 **
 ******************************************************************************/
en_result_t Lvd_DisableFunc(en_lvd_func_t enFunc)
{ 
    FM0P_LVD->LVD_RLR = LvdCtlUnlock;/* unlock LVD_CTL */     
    switch(enFunc)
    {
        case LvdIntFunc:
            FM0P_LVD->LVD_CTL_f.LVDIE = 0;
            break;
        case LvdResetFunc:
            FM0P_LVD->LVD_CTL_f.LVDRE = 0;
            break;  
        default:
            FM0P_LVD->LVD_RLR = 0;           /* lock LVD_CTL */
            return  ErrorInvalidParameter;
    }
    
    FM0P_LVD->LVD_RLR = 0;           /* lock LVD_CTL */
    
    return Ok;
}

/**
 ******************************************************************************
 ** \brief Get the LVD interrupt status
 ** 
 ** \retval FALSE       A low-voltage detection is not detected.
 ** \retval TRUE        A low-voltage detection has been detected.
 ** 
 ******************************************************************************/
boolean_t Lvd_GetIntStatus(void)
{
    boolean_t bRetVal = FALSE;

    /* Check the LVD interrupt status */
    if (TRUE == FM0P_LVD->LVD_STR_f.LVDIR)
    {
        bRetVal = TRUE;
    }
    
    return (bRetVal);
} /* Lvd_GetIntOperationStatus */

/**
 ******************************************************************************
 ** \brief Clear the LVD interrupt status
 **
 ******************************************************************************/
void Lvd_ClearIntStatus(void)
{
    FM0P_LVD->LVD_CLR = 0x00;  /* Clear LVD interrupt (LVDCL (bit #7)) */
} /* Lvd_ClearIntStatus */


//@} // LvdGroup

#endif /* #if (defined(PDL_PERIPHERAL_LVD_ACTIVE)) */

/******************************************************************************/
/* EOF (not truncated)                                                        */
/******************************************************************************/
