/*******************************************************************************
* Copyright (C) 2013 Spansion LLC. All Rights Reserved. 
*
* This software is owned and published by: 
* Spansion LLC, 915 DeGuigne Dr. Sunnyvale, CA  94088-3453 ("Spansion").
*
* BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND 
* BY ALL THE TERMS AND CONDITIONS OF THIS AGREEMENT.
*
* This software contains source code for use with Spansion 
* components. This software is licensed by Spansion to be adapted only 
* for use in systems utilizing Spansion components. Spansion shall not be 
* responsible for misuse or illegal use of this software for devices not 
* supported herein.  Spansion is providing this software "AS IS" and will 
* not be responsible for issues arising from incorrect user implementation 
* of the software.  
*
* SPANSION MAKES NO WARRANTY, EXPRESS OR IMPLIED, ARISING BY LAW OR OTHERWISE,
* REGARDING THE SOFTWARE (INCLUDING ANY ACOOMPANYING WRITTEN MATERIALS), 
* ITS PERFORMANCE OR SUITABILITY FOR YOUR INTENDED USE, INCLUDING, 
* WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY, THE IMPLIED 
* WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE OR USE, AND THE IMPLIED 
* WARRANTY OF NONINFRINGEMENT.  
* SPANSION SHALL HAVE NO LIABILITY (WHETHER IN CONTRACT, WARRANTY, TORT, 
* NEGLIGENCE OR OTHERWISE) FOR ANY DAMAGES WHATSOEVER (INCLUDING, WITHOUT 
* LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS, BUSINESS INTERRUPTION, 
* LOSS OF BUSINESS INFORMATION, OR OTHER PECUNIARY LOSS) ARISING FROM USE OR 
* INABILITY TO USE THE SOFTWARE, INCLUDING, WITHOUT LIMITATION, ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL OR CONSEQUENTIAL DAMAGES OR LOSS OF DATA, 
* SAVINGS OR PROFITS, 
* EVEN IF SPANSION HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. 
* YOU ASSUME ALL RESPONSIBILITIES FOR SELECTION OF THE SOFTWARE TO ACHIEVE YOUR
* INTENDED RESULTS, AND FOR THE INSTALLATION OF, USE OF, AND RESULTS OBTAINED 
* FROM, THE SOFTWARE.  
*
* This software may be replicated in part or whole for the licensed use, 
* with the restriction that this Disclaimer and Copyright notice must be 
* included with each copy of this software, whether used in part or whole, 
* at all times.  
*/
/******************************************************************************/
/** \file rtc.h
 **
 ** Headerfile for RTC functions
 **  
 **
 ** History:
 **   - 2013-02-21   Edison Zhang      First verison
 **
 ******************************************************************************/

#ifndef __RTC_H__
#define __RTC_H__

/******************************************************************************/
/* Include files                                                              */
/******************************************************************************/
#include "mcu.h"
#include "pdl_user.h"
#include "time.h"

#if (defined(PDL_PERIPHERAL_RTC_ACTIVE))

/* C binding of definitions if building with C++ compiler */
#ifdef __cplusplus
extern "C"
{
#endif

/**
 ******************************************************************************
 ** \defgroup RtcGroup Real Time Clock (RTC)
 **
 ** Provided RTC module functions:
 **
 ** - Rtc_Init()
 ** - Rtc_DeInit()
 ** - Rtc_EnableInt()
 ** - Rtc_DisableInt()
 ** - Rtc_IrqHandler()
 ** - Rtc_EnableFunc()
 ** - Rtc_DisableFunc()
 ** - Rtc_Reset()
 ** - Rtc_GetStatus()
 ** - Rtc_ClrStatus()
 ** - Rtc_SetDateTime()
 ** - Rtc_ReadDateTime()
 ** - Rtc_SetAlarmDateTime()
 ** - Rtc_GetAlarmDateTime()
 ** - Rtc_SetTimerCycle()
 ** - Rtc_SetFreqCorrValue()
 ** - Rtc_SetDayOfWeek()
 **
 ** Rtc_Init() initializes an RTC instance according the configuration
 ** of the structure type #stc_rtc_config_t. Rtc_DeInit() de-initializes an
 ** RTC instance. 
 **
 ** Rtc_EnableInt() enables RTC interrupt sources selected by parameter
 ** Rtc_EnableInt#stc_rtc_int_sel_t and sets the according interrupt 
 ** callback functions in the internal data structure. Rtc_DisableInt()
 ** disables the RTC interrupt sources selected.
 **
 ** Rtc_IrqHandler() is RTC IRQ handler function, which is called in 
 ** interrupts.c. 
 **
 ** Rtc_EnableFunc() enables RTC function selected by the parameter 
 ** Rtc_EnableFunc#enFunc and Rtc_DisableFunc() disables RTC function selected.
 **
 ** Rtc_Reset() resets RTC peripheral as well as RTC registers.
 **
 ** Rtc_GetStatus() gets the status selected by Rtc_GetStatus#enStatus
 ** and Rtc_ClrStatus() clears the RTC status selected, some status can
 ** only be cleared by hardware automatically. 
 **
 ** Rtc_SetDateTime() can change the time and date of RTC and Rtc_GetDateTime()
 ** gets current time and date. 
 **
 ** Rtc_SetAlarmDateTime() can change the alarm time and date, and
 ** Rtc_GetAlarmDateTime() gets the alarm time and date.
 **
 ** Rtc_SetTimerCycle() sets the cycle of RTC timer, note that the unit of cycle
 ** is second.
 **
 ** Rtc_SetFreqCorrValue() sets the frequency correction value of RTC. 
 **
 ** Rtc_SetDayOfWeek() sets the day of week based on time structure supplied. 
 ** 
 ******************************************************************************/
//@{

/**
 ******************************************************************************
 ** \brief Rtccallback function prototypes.
 ******************************************************************************/

/******************************************************************************
 * Global definitions
 ******************************************************************************/
/**
 ******************************************************************************
 ** \brief Maximum Frequency Correction Value
 ******************************************************************************/  
#define RTC_MAX_FREQ_CORR_VALUE (0x3FFu)

/**
 ******************************************************************************
 ** \brief Maximum Frequency Correction Cycle Setting Value
 ******************************************************************************/  
#define RTC_MAX_FREQ_CORR_CYCLE_SET_VALUE (0x3Fu)

/**
 ******************************************************************************
 ** \brief Maxmimum Timer Setting Value [unit: second]
 ******************************************************************************/  
#define RTC_MAX_TIMER_SET_VALUE (0x20000)

/******************************************************************************
 * Global type definitions
 ******************************************************************************/
/**
 ******************************************************************************
 ** \brief Real Time Clock clock selection
 ******************************************************************************/
typedef enum en_rtc_clk_sel
{
    RtcUseSubClk  = 0u,      ///< Use sub clock as RTC source clock
    RtcuseMainClk = 1u,      ///< Use main clock as RTC source clock
  
}en_rtc_clk_sel_t;

/**
 ******************************************************************************
 ** \brief RTCCO output selection
 **
 ** If user needs to output RTCCO, RTCCO pin function should also be enabled in 
 ** in GPIO macro.
 ******************************************************************************/
typedef enum en_rtc_rtcco_sel
{
    RtccoOutput2Hz = 0,    ///< Output 2Hz signal at RTCCO pin
    RtccoOutput1Hz = 1,    ///< Output 1Hz signal at RTCCO pin
    
}en_rtc_rtcco_sel_t;

/**
 ******************************************************************************
 ** \brief frequency correction configuration
 ******************************************************************************/
typedef struct stc_rtc_freq_corr
{ 
    uint16_t u16FreqCorrValue;  ///< Frequency correction value
    uint16_t u16FreqCorrCycle;  ///< Frequency correction cycle
    
}stc_rtc_freq_corr_t;

/**
 ******************************************************************************
 ** \brief Real Time Clock functions
 ******************************************************************************/
typedef enum en_rtc_func
{
    RtcCount         = 0u,    ///< RTC counting
    RtcTimer         = 1u,    ///< RTC timer
    RtcFreqCorr      = 2u,    ///< RTC frequency correction module
    RtcAlarmYearEn   = 3u,    ///< Year comparison enable of RTC alarm
    RtcAlarmMonthEn  = 4u,    ///< Month comparison enable of RTC alarm
    RtcAlarmDayEn    = 5u,    ///< D comparison enable of RTC alarm
    RtcAlarmHourEn   = 6u,    ///< Hour comparison enable of RTC alarm 
    RtcAlarmMinEn    = 7u,    ///< Minute comparison enable of RTC alarm 
    
}en_rtc_func_t;

/**
 ******************************************************************************
 ** \brief Day of week name definitions 
 ******************************************************************************/
typedef enum en_rtc_day_of_week
{
    RtcSunday    = 0,     ///< Sunday
    RtcMonday    = 1,     ///< Monday
    RtcTuesday   = 2,     ///< Tuesday
    RtcWednesday = 3,     ///< Wednesday
    RtcThursday  = 4,     ///< Thursday
    RtcFriday    = 5,     ///< Friday
    RtcSaturday  = 6      ///< Saturday
} en_rtc_day_of_week_t;

/**
 ******************************************************************************
 ** \brief Month name definitions (not used in driver - to be used by
 **        user appliciation)
 ******************************************************************************/
typedef enum en_rtc_month
{
    RtcJanuary   = 1,   ///< January
    RtcFebuary   = 2,   ///< February 
    RtcMarch     = 3,   ///< March
    RtcApril     = 4,   ///< April
    RtcMay       = 5,   ///< May 
    RtcJune      = 6,   ///< June
    RtcJuly      = 7,   ///< July
    RtcAugust    = 8,   ///< August
    RtcSeptember = 9,   ///< September
    RtcOctober   = 10,  ///< October 
    RtcNovember  = 11,  ///< November
    RtcDecember  = 12   ///< December
} en_rtc_month_t;
      
/**
 ******************************************************************************
 ** \brief Real Time Clock timer mode selection
 ******************************************************************************/
typedef enum en_rtc_time_mode
{
    RtcTimerOneshot = 0u,  ///< One-shot mode
    RtcTimerPeriod  = 1u,  ///< Periodical mode
    
}en_rtc_time_mode_t;

/**
 ******************************************************************************
 ** \brief Real Time Clock timer configuration structure
 ******************************************************************************/
typedef struct stc_rtc_timer
{
    en_rtc_time_mode_t       enMode;            ///< Timer mode
    uint32_t                 u32TimerCycle;     ///< The cycle [uint: second] of timer, after these seconds elapse, timer interrpt will occur
    
}stc_rtc_timer_t;

/**
 ******************************************************************************
 ** \brief Rtc Divider Ratio Setting
 ** 
 ** Divider Ration Settings for WTC. The enumarted values do not correspond to
 ** the bit patterns of WTDIV!
 ******************************************************************************/
typedef enum en_rtc_div_ratio
{
    RtcDivRatio1     = 0x00u,    ///< RIN clock is not divided
    RtcDivRatio2     = 0x01u,    ///< RIN clock is divided by 2
    RtcDivRatio4     = 0x02u,    ///< RIN clock is divided by 4
    RtcDivRatio8     = 0x03u,    ///< RIN clock is divided by 8
    RtcDivRatio16    = 0x04u,    ///< RIN clock is divided by 16
    RtcDivRatio32    = 0x05u,    ///< RIN clock is divided by 32
    RtcDivRatio64    = 0x06u,    ///< RIN clock is divided by 64
    RtcDivRatio128   = 0x07u,    ///< RIN clock is divided by 128
    RtcDivRatio256   = 0x08u,    ///< RIN clock is divided by 256
    RtcDivRatio512   = 0x09u,    ///< RIN clock is divided by 512
    RtcDivRatio1024  = 0x0Au,    ///< RIN clock is divided by 1024
    RtcDivRatio2048  = 0x0Bu,    ///< RIN clock is divided by 2048
    RtcDivRatio4096  = 0x0Cu,    ///< RIN clock is divided by 4096
    RtcDivRatio8192  = 0x0Du,    ///< RIN clock is divided by 8192
    RtcDivRatio16384 = 0x0Eu,    ///< RIN clock is divided by 16384
    RtcDivRatio32768 = 0x0Fu,    ///< RIN clock is divided by 32768
} en_rtc_div_ratio_t ;

/**
 ******************************************************************************
 ** \brief Interrupt configuration structure
 ******************************************************************************/
typedef struct stc_rtc_int_sel
{
    uint8_t  HalfSecondInt         : 1; ///< 1: 0.5-Second interrupt enabled
                                        ///< 0: 0.5-Second interrupt disabled
    uint8_t  OneSecondInt          : 1; ///< 1: 1-Second interrupt enabled
                                        ///< 0: 1-Second interrupt disabled
    uint8_t  OneMinuteInt          : 1; ///< 1: 1-Minute interrupt enabled
                                        ///< 0: 1-Minute interrupt disabled
    uint8_t  OneHourInt            : 1; ///< 1: 1-Hour interrupt enabled
                                        ///< 0: 1-Hour interrupt disabled
    uint8_t  TimerInt              : 1; ///< 1: Timer interrupt enabled
                                        ///< 0: Timer interrupt disabled  
    uint8_t  AlarmInt              : 1; ///< 1: Alarm interrupt enabled
                                        ///< 0: Alarm interrupt disabled
    uint8_t  TimeRewriteErrorInt   : 1; ///< 1: Time Rewrite Error interrupt enabled
                                        ///< 0: Time Rewrite Error interrupt disabled
} stc_rtc_int_sel_t;

/**
 ******************************************************************************
 ** \brief RTC callback functions structure
 ******************************************************************************/
typedef struct stc_rtc_int_cb
{
    func_ptr_t               pfnTimeWrtErrCallback; ///< Callback function pointer for Timer writing error Interrupt
    func_ptr_t               pfnAlarmCallback;      ///< Callback function pointer for Alarm Interrupt
    func_ptr_t               pfnTimerCallback;      ///< Callback function pointer for Timer Interrupt
    func_ptr_t               pfnHalfSecondCallback; ///< Callback function pointer for 0.5-Second Interrupt
    func_ptr_t               pfnOneSecondCallback;  ///< Callback function pointer for One Second Interrupt
    func_ptr_t               pfnOneMinuteCallback;  ///< Callback function pointer for One Minute Interrupt
    func_ptr_t               pfnOneHourCallback;    ///< Callback function pointer for One Hour Interrupt
    
}stc_rtc_int_cb_t;

/**
 ******************************************************************************
 ** \brief Real Time Clock Date and Time structure
 ******************************************************************************/
typedef struct stc_rtc_time
{
    uint8_t  u8Second;      ///< Second (Format 0...59)
    uint8_t  u8Minute;      ///< Minutes (Format 0...59)
    uint8_t  u8Hour;        ///< Hour (Format 0...23)
    uint8_t  u8Day;	        ///< Day (Format 1...31)
    uint8_t  u8DayOfWeek;   ///< Day of the week (Format 0...6)
    uint8_t  u8Month;       ///< Month (Format 1...12)
    uint16_t u16Year;       ///< Year (Format 1...99) + 2000
} stc_rtc_time_t;

/**
 ******************************************************************************
 ** \brief Real Time Clock status types
 ******************************************************************************/
typedef enum en_rtc_status
{
    RtcRewriteError          = 0u,  ///< Rewrite error 
    RtcAlarmElementMatch     = 1u,  ///< One of alarm element matchs
    RtcTimerUnderFlow        = 2u,  ///< Timer underflow
    RtcOneHourFlag           = 3u,  ///< 1-hour count-up flag
    RtcOneMiniteFlag         = 4u,  ///< 1-Minute count-up flag
    RtcOneSecondFlag         = 5u,  ///< 1-second count-up flag
    RtcHalfSecondFlag        = 6u,  ///< 0.5-second count-up flag
    RtcRunStatus             = 7u,  ///< RTC counter run status
    RtcTimerStatus           = 8u,  ///< RTC timer status
    
}en_rtc_status_t;    
    
/**
 ******************************************************************************
 ** \brief Real Time Clock Alarm structure
 ******************************************************************************/
typedef struct stc_rtc_alarm
{
    uint8_t  u8Minute; ///< Alarm minutes (Format 1...59)
    uint8_t  u8Hour;   ///< Alarm hour (Format 0...23)  
    uint8_t  u8Day;    ///< Alarm day (Format 1...31) 
    uint8_t  u8Month;  ///< Alarm month (Foramt 1...12)
    uint16_t u16Year;  ///< Alarm year (Format 1...99) + 2000  
} stc_rtc_alarm_t;

/**
 ******************************************************************************
 ** \brief Real Time Clock configuration
 ** 
 ** The RTC configuration settings
 ******************************************************************************/
typedef struct stc_rtc_config
{
    en_rtc_clk_sel_t         enClkSel;             ///< TRUE:  RTC clock selection 
    boolean_t                bEnSuboutDivider;     ///< TRUE:  Enable Divider for Divider Ratio
                                                   ///< FALSE: Disable Divider for Divider Ratio
    en_rtc_div_ratio_t       enDividerRatio;       ///< Divider ratio
    en_rtc_rtcco_sel_t       enRtccoSel;           ///< Divider of CO signal, see #en_rtc_rtcco_sel_t for details
    
    boolean_t                bInitFreqCorr;        ///< TRUE:  Init Frequency correction module
                                                   ///< FALSE: Do not Init Frequency correction module
    stc_rtc_freq_corr_t      stcFreqCorrConfig;    ///< Frequency correction configuration, only when bInitFreqCorr is TRUE, this configuration is valid.     
    
    boolean_t                bInitTimeDate;        ///< TRUE:  Init RTC time and date
                                                   ///< FALSE: Do not Init RTC time and date
    stc_rtc_time_t           stcTimeDate;          ///< Time configuration structure, only when bInitTime is TRUE, this configuration is valid.  
    
    boolean_t                bInitAlarm;           ///< TRUE:  Init RTC alarm
                                                   ///< FALSE: Do not Init RTC alarm
    stc_rtc_alarm_t          stcAlarm;             ///< Alarm configuration structure, only when bInitAlarm is TRUE, this configuration is valid. 
    
    boolean_t                bInitTimer;           ///< TRUE:  Init RTC timer
                                                   ///< FALSE: Do not Init RTC timer
    stc_rtc_timer_t          stcTimer;             ///< Timer configuration structure, only when bInitTimer is TRUE, this configuration is valid. 

} stc_rtc_config_t;


/******************************************************************************/
/* Local type definitions ('typedef')                                         */
/******************************************************************************/
/// Datatype for holding internal data needed for RTC
typedef struct stc_rtc_intern_data
{
    func_ptr_t             pfnTimeWrtErrCallback;   ///< Callback function pointer Timer writing error Interrupt
    func_ptr_t             pfnAlarmCallback;        ///< Callback function pointer Alarm Interrupt
    func_ptr_t             pfnTimerCallback;        ///< Callback function pointer Timer Interrupt
    func_ptr_t             pfnHalfSecondCallback;   ///< Callback function pointer 0.5-Second Interrupt
    func_ptr_t             pfnOneSecondCallback;    ///< Callback function pointer One Second Interrupt
    func_ptr_t             pfnOneMinuteCallback;    ///< Callback function pointer One Minute Interrupt
    func_ptr_t             pfnOneHourCallback;      ///< Callback function pointer One Hour Interrupt

} stc_rtc_intern_data_t ;

/******************************************************************************/
/* Global function prototypes (definition in C source)                        */
/******************************************************************************/
#if (PDL_INTERRUPT_ENABLE_RTC == PDL_ON)
void Rtc_IrqHandler(void);
en_result_t Rtc_EnableInt(stc_rtc_int_sel_t* pstcIntSel, stc_rtc_int_cb_t* pstcIntCb);
en_result_t Rtc_DisableInt(stc_rtc_int_sel_t* pstcIntSel);
#endif

/* Init/De-Init */
en_result_t Rtc_Init(stc_rtc_config_t*       pstcConfig);
en_result_t Rtc_DeInit(void);

/* Function enable/disable */
en_result_t Rtc_EnableFunc(en_rtc_func_t enFunc);
en_result_t Rtc_DisableFunc(en_rtc_func_t enFunc);

/* Rtc Reset */
void Rtc_Reset(void);

/* Get/Clr RTC status */
boolean_t Rtc_GetStatus(en_rtc_status_t enStatus);
en_result_t Rtc_ClrStatus(en_rtc_status_t enStatus);

/* Calendar/Alarm set/get */
en_result_t Rtc_SetDateTime(stc_rtc_time_t* pstcTimeDate, boolean_t bContinue);
en_result_t Rtc_ReadDateTime(stc_rtc_time_t* pstcTimeDate);
en_result_t Rtc_SetAlarmDateTime(stc_rtc_alarm_t* pstcAlarm);
en_result_t Rtc_GetAlarmDateTime(stc_rtc_alarm_t* pstcAlarm);

/* Set timer cycle */
en_result_t Rtc_SetTimerCycle(uint32_t u32TimerCycle); 

/* Set frequency correction value */
en_result_t Rtc_SetFreqCorrValue(uint16_t u16Value); 

/* Set day of week */
en_result_t Rtc_SetDayOfWeek(stc_rtc_time_t* pstcRtcTime);


//@} // RtcGroup

#ifdef __cplusplus
}
#endif

#endif /* #if (defined(PDL_PERIPHERAL_RTC_ACTIVE)) */

#endif /* __RTC_H__ */
/******************************************************************************/
/* EOF (not truncated)                                                        */
/******************************************************************************/
